import numpy as np
from sklearn.ensemble import GradientBoostingRegressor
from sklearn.metrics import mean_squared_error
from sklearn.model_selection import KFold
from sklearn.preprocessing import StandardScaler
import pickle, scipy, h5py, argparse, sys
from random import randrange, seed

parser = argparse.ArgumentParser(description='GBDT inputs')
parser.add_argument('--n_estimators', default=20000, type=int, help='Maximum tree depth')
parser.add_argument('--dataset', default='S8338', type=str, help='Dataset selected')
parser.add_argument('--datatype', default='TopSpe', type=str, help='Dataset selected')
parser.add_argument('--PGS0', default='PGS0_0', type=str, help='for PGS0 test')
parser.add_argument('--max_depth', default=9, type=int, help='Maximum tree depth')
parser.add_argument('--learning_rate', default=0.005, type=float, help='Learning rate for gbrt')
parser.add_argument('--criterion', default='friedman_mse', type=str, help='Loss function for gbrt')
parser.add_argument('--subsample', default=0.4, type=float, help='Subsample for fitting individual learners')
parser.add_argument('--max_features', default='sqrt', type=str, help='Number of features to be considered')
parser.add_argument('--min_samples_split', default=3, type=int, help='Minimum sample num of each leaf node.')
parser.add_argument('--loss', default='ls', type=str, help='Loss function to be optimized.')
parser.add_argument('--n_iter_no_change', default=500, type=int, help='Early stopping will be used to terminate training')
parser.add_argument('--random_seed', default=0, type=int, help='random seed')
parser.add_argument('--iteration', default=1, type=int, help='random seed')
parser.add_argument('--n_splits', default=10, type=int, help='# fold')
args = parser.parse_args()

X_val = np.load('X_val_'+args.dataset+'.npy')
y_val = np.load('y_val_'+args.dataset+'.npy')
print('The data shape', X_val.shape)


print('GBDT parameter')
print(args)
for i in range(args.iteration):
    results = np.zeros(y_val.shape)
    kf = KFold(n_splits=args.n_splits, shuffle=True, random_state=i)
    for j, [train_idx, test_idx] in enumerate(kf.split(X_val)):
        X_train, X_test = X_val[train_idx], X_val[test_idx]
        y_train, y_test = y_val[train_idx], y_val[test_idx]

        print('>>>>>>>>>>>>>training.............')
        # gradient boosting regressor
        GBR = GradientBoostingRegressor(n_estimators      = args.n_estimators, \
                                        learning_rate     = args.learning_rate, \
                                        max_features      = args.max_features, \
                                        max_depth         = args.max_depth, \
                                        min_samples_split = args.min_samples_split, \
                                        subsample         = args.subsample, \
                                        n_iter_no_change  = args.n_iter_no_change, \
                                        criterion         = args.criterion, \
                                        loss              = args.loss)
        GBR.fit(X_train, y_train)

        print('>>>>>>>>>>>>>predict.............')
        # gradient boosting regressor prediction
        y_pred = GBR.predict(X_test)
        results[test_idx] = y_pred

        RMSD = np.sqrt(mean_squared_error(y_test, y_pred))
        pearsonr = scipy.stats.pearsonr(y_test, y_pred)

        print('RMSD: %f, Pearsonr: %f'%(RMSD, pearsonr[0]))

    RMSD = np.sqrt(mean_squared_error(y_val, results))
    pearsonr = scipy.stats.pearsonr(y_val, results)

    print('%d-th Final RMSD: %f, Pearsonr %f'%(i, RMSD, pearsonr[0]))
